/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.productionunit;

import com.vividsolutions.jts.geom.Geometry;
import java.util.Date;
import nl.cloudfarming.client.farm.model.CropField;
import nl.cloudfarming.client.farm.model.CropProductionUnit;
import nl.cloudfarming.client.geoviewer.SplitGeometryHandler;
import nl.cloudfarming.client.lib.geotools.GeometryTools;
import org.openide.nodes.Node;

/**
 * Split handler for {@link CropField}
 * @author johan
 */
public class CropFieldSplitHandler extends SplitGeometryHandler {
    private final CropField sourceCropField;

    public CropFieldSplitHandler(CropField cropField) {
        this.sourceCropField = cropField;
    }

    @Override
    public Geometry getSource() {
        return sourceCropField.getGeometry();
    }

    @Override
    public boolean handle(Geometry[] geometries, Node sourceNode) {
        CropProductionUnit cropProductionUnit = sourceCropField.getParent();
        
        if (cropProductionUnit == null) {
            //TODO: log message in GUI
            return false;
        }

        for (int i = 0; i < geometries.length; i++) {
            // create new cropfield based on original cropfield and new geometry,
            CropField newCropField = new CropField(sourceCropField);
            newCropField.setGeometry(geometries[i]);
            newCropField.setAreaInHectares(GeometryTools.getAreaInHA(geometries[i]));
            newCropField.setName(sourceCropField.getName() + "-part-" + i);
            
            // add to same CropProductionUnit as original:
            cropProductionUnit.addCropField(newCropField);
            
            addToMap(new CropFieldNode(newCropField));
        }
        //mark the original CropField as expired by setting it's end date
        sourceCropField.setEndDate(new Date());
        removeFromMap(sourceNode);
        
        return true;
    }
    
}
