/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.productionunit;

import java.awt.datatransfer.Transferable;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.farm.model.CropProductionUnit;
import org.openide.util.datatransfer.PasteType;

/**
 * <p>DropTarget listener for the {@link CropProductionUnit}</p>
 * Use to add as DropTargetListener on your panel. Gets the appropriate pasteType from {@link CropProductionUnitDropType} 
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
public class CropProductionUnitDropTargetListener implements DropTargetListener {

    private CropProductionUnit cropProductionUnit;
    protected static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.productionunit");

    public CropProductionUnitDropTargetListener(CropProductionUnit cropProductionUnit) {
        this.cropProductionUnit = cropProductionUnit;
    }

    @Override
    public void dragEnter(DropTargetDragEvent dtde) {
    }

    @Override
    public void dragOver(DropTargetDragEvent dtde) {
    }

    @Override
    public void dropActionChanged(DropTargetDragEvent dtde) {
    }

    @Override
    public void dragExit(DropTargetEvent dte) {
    }

    @Override
    public void drop(DropTargetDropEvent dtde) {
        Transferable transferable = dtde.getTransferable();

        PasteType pasteType = CropProductionUnitPasteType.getPasteType(transferable, cropProductionUnit);
        if(pasteType != null){
            try {
                pasteType.paste();
            } catch (IOException ex) {
                LOGGER.log(Level.WARNING, "IOException occurred while trying to paste, {0}", transferable.toString());
            }
        }
    }
}
