/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.productionunit;

import java.awt.Image;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Action;
import nl.cloudfarming.client.farm.model.CropField;
import nl.cloudfarming.client.farm.model.CropProductionUnit;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.nodes.NodeTransfer;
import org.openide.util.ImageUtilities;
import org.openide.util.Utilities;
import org.openide.util.datatransfer.PasteType;

/**
 * Node which represents a {@link CropProductionUnit}
 *
 * @author Maciek Dulkiewicz <maciek.dulkiewicz@ordina.nl>
 */
public final class CropProductionUnitNode extends FilterNode implements Serializable {

    private static final String CROP_PRODUCTION_UNIT_ACTION_PATH = "Actions/Productionunit";
    private static final String ICON = "nl/cloudfarming/client/productionunit/green16.png";
    protected CropProductionUnit cropProductionUnit;
    protected static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.productionunit");

    public CropProductionUnitNode(Node original, CropProductionUnit cropProductionUnit) {
        super(original, Children.create(new CropProductionUnitChildFactory(cropProductionUnit), true));
        this.cropProductionUnit = cropProductionUnit;
        
        disableDelegation(DELEGATE_SET_NAME);
        disableDelegation(DELEGATE_GET_NAME);
        setName(cropProductionUnit.getName());
        cropProductionUnit.addPropertyChangeListener(CropProductionUnit.PROP_NAME, new PropertyChangeListener() {

            @Override
            public void propertyChange(PropertyChangeEvent pce) {
                setName((String)pce.getNewValue());
                setDisplayName(getName());
            }
        });
    }

    @Override
    public String getDisplayName() {
        return cropProductionUnit.getName();
    }

    @Override
    public Image getIcon(int type) {
        return ImageUtilities.loadImage(ICON);
    }

    @Override
    public Image getOpenedIcon(int type) {
        return getIcon(type);
    }

    @Override
    public boolean canCopy() {
        return true;
    }

    @Override
    public boolean canCut() {
        return false;
    }

    @Override
    public boolean canDestroy() {
        return true;
    }

    @Override
    public Transferable clipboardCopy() throws IOException {
        // copy the filternode itself instead of delegating to the the original datanode
        return NodeTransfer.transferable(this, NodeTransfer.CLIPBOARD_COPY);
    }

    @Override
    public Transferable clipboardCut() throws IOException {
        return clipboardCopy();
    }

    @Override
    public Transferable drag() throws IOException {
        return clipboardCopy();
    }

    @Override
    public boolean canRename() {
        return true;
    }

    @Override
    public void setName(String name) {
        super.setName(name);
        cropProductionUnit.setName(name);
        setDisplayName(name);
    }

    @Override
    public Action[] getActions(boolean context) {
        List<Action> actions = new ArrayList();
        actions.addAll(Arrays.asList(super.getActions(context)));
        for (Action action : Utilities.actionsForPath(CROP_PRODUCTION_UNIT_ACTION_PATH)) {
            actions.add(action);
        }
        return actions.toArray(new Action[actions.size()]);
    }

    @Override
    public PasteType getDropType(final Transferable transferable, int action, int index) {
        return CropProductionUnitPasteType.getPasteType(transferable, cropProductionUnit);        
    }
}
