/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.productionunit.wizard;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JComponent;
import nl.cloudfarming.client.crop.model.CropInstance;
import nl.cloudfarming.client.farm.model.CropProductionUnit;
import nl.cloudfarming.client.farm.model.CropProductionUnitManager;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import org.netbeans.api.project.Project;
import org.openide.DialogDisplayer;
import org.openide.WizardDescriptor;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

@NbBundle.Messages({"CropProductionUnitWizardAction_wizard_title=New production unit"})
public final class CropProductionUnitWizardAction implements ActionListener {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.productionunit");
    public static final String CPU = "cpu";
    private Project project;
    private WizardDescriptor wiz;

    public CropProductionUnitWizardAction(Project project) {
        this.project = project;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        List<WizardDescriptor.Panel<WizardDescriptor>> panels = new ArrayList<>();
        panels.add(new CropProductionUnitInputPanel(createProductionUnit()));
        panels.add(new CropProductionUnitSummaryPanel());
        String[] steps = new String[panels.size()];
        for (int i = 0; i < panels.size(); i++) {
            Component c = panels.get(i).getComponent();
            // Default step name to component name of panel.
            steps[i] = c.getName();
            if (c instanceof JComponent) { // assume Swing components
                JComponent jc = (JComponent) c;
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_SELECTED_INDEX, i);
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_DATA, steps);
                jc.putClientProperty(WizardDescriptor.PROP_AUTO_WIZARD_STYLE, true);
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_DISPLAYED, true);
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_NUMBERED, true);
            }
        }
        wiz = new WizardDescriptor(new WizardDescriptor.ArrayIterator<>(panels));
        // {0} will be replaced by WizardDesriptor.Panel.getComponent().getName()
        wiz.setTitleFormat(new MessageFormat("{0}"));
        wiz.setTitle(Bundle.CropProductionUnitWizardAction_wizard_title());
        if (DialogDisplayer.getDefault().notify(wiz) == WizardDescriptor.FINISH_OPTION) {
            CropProductionUnit productionUnit = (CropProductionUnit) wiz.getProperty(CropProductionUnitWizardAction.CPU);
            saveProductionUnit(productionUnit);
        }
    }

    /**
     * Creates an empty {@link CropProductionUnit} without an {@link CropInstance}
     * Sets the start date to today
     * @return 
     */
    private CropProductionUnit createProductionUnit() {
        EnterpriseIdProvider idProvider = project.getLookup().lookup(EnterpriseIdProvider.class);
        CropProductionUnit productionUnit = new CropProductionUnit(idProvider);
        productionUnit.setStartDate(new Date());
        return productionUnit;
    }

    /**
     * Saves the provided CropProductionUnit
     * @param cropProductionUnit 
     */
    private void saveProductionUnit(CropProductionUnit cropProductionUnit) {
        CropProductionUnitManager manager = Lookup.getDefault().lookup(CropProductionUnitManager.class);
        if (manager != null) {
            manager.save(cropProductionUnit);
        } else {
            LOGGER.log(Level.WARNING, "CropProductionManager not found, could not save crop production unit.");
        }
    }
}
